/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.database;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.provider.BaseColumns;
import android.text.format.DateUtils;
import android.util.Log;

/**
 * Extremely simple cache implementation to allow storing key/value pairs to
 * the SQLite database.
 * 
 * @author Ian G. Clifton
 */
public class SimpleCache {
	
	private final long DEFAULT_EXPIRATION = DateUtils.MINUTE_IN_MILLIS * 5;
	
	private final String mKey;
	private final long mTimestamp;
	private final String mValue;
	
	/**
	 * Constructs a new SimpleCache with the current system time as the timestamp
	 * 
	 * @param key String key to identify this cache
	 * @param value String value to store
	 */
	public SimpleCache(String key, String value) {
		this(key, value, System.currentTimeMillis());
	}
	
	/**
	 * Constructs a new SimpleCache with the specified values
	 * 
	 * @param key String key to identify this cache
	 * @param value String value to store
	 * @param timestamp long timestamp for when this cache was originally created
	 */
	public SimpleCache(String key, String value, long timestamp) {
		mKey = key;
		mValue = value;
		mTimestamp = timestamp;
	}

	/**
	 * Returns value stored in this SimpleCache
	 * 
	 * @return value stored in this SimpleCache
	 */
	public String getValue() {
		return mValue;
	}
	
	/**
	 * Returns true if the cache has expired
	 * 
	 * @return true if the cache has expired
	 */
	public boolean hasExpired() {
		return System.currentTimeMillis() > (mTimestamp + DEFAULT_EXPIRATION);
	}
	
	/**
	 * Static method returns a SimpleCache object from the database
	 * 
	 * @param context Context for accessing the database
	 * @param key String to identify the SimpleCache to load
	 * @return SimpleCache object from the database, null when not found
	 */
	public static SimpleCache loadSimpleCache(Context context, String key) {
		DatabaseHelper dh = new DatabaseHelper(context);
		SimpleCache result = dh.loadSimpleCache(key);
		dh.close();
		return result;
	}
	
	/**
	 * Saves the SimpleCache object to the database
	 * 
	 * @param context Context for accessing the database
	 * @return true on success
	 */
	public boolean save(Context context) {
		DatabaseHelper dh = new DatabaseHelper(context);
		boolean result = dh.replaceSimpleCache(this);
		dh.close();
		return result;
	}
	
	private static class DatabaseHelper extends SQLiteOpenHelper implements BaseColumns {

		private static final String COLUMN_KEY = "key";
		private static final String COLUMN_TIMESTAMP = "timestamp";
		private static final String COLUMN_VALUE = "value";
		private static final String DATABASE_NAME = "simplecache.db";
		private static final int DATABASE_VERSION = 1;
		private static final String TABLE_NAME = "simplecache";
		
		private static final String CREATE_TABLE = "CREATE TABLE " + TABLE_NAME +
				" (" +
				_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, " + 
				COLUMN_KEY + " TEXT UNIQUE NOT NULL, " +
				COLUMN_VALUE + " TEXT NOT NULL, " +
				COLUMN_TIMESTAMP + " INTEGER NOT NULL" +
				");";
		
		public DatabaseHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(CREATE_TABLE);
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			db.execSQL("DROP TABLE IF EXISTS " + TABLE_NAME);
			onCreate(db);
		}
		
		public SimpleCache loadSimpleCache(String key) {
			final SQLiteDatabase db;
			try {
				db = getReadableDatabase();
			} catch (SQLiteException e) {
				Log.w(SimpleCache.class.getName(), "Failed to get readable database: " + e.getMessage());
				return null;
			}
			
			String[] columns = new String[] {COLUMN_KEY, COLUMN_VALUE, COLUMN_TIMESTAMP};
			String selection = COLUMN_KEY + "=?";
			
			Cursor cursor = null;
			try {
				cursor = db.query(TABLE_NAME, columns, selection, new String[] {key}, null, null, null);
				if (cursor == null) {
					return null;
				}
				
				if (!cursor.moveToFirst()) {
					cursor.close();
					return null;
				}
				
				String returnedKey = cursor.getString(cursor.getColumnIndex(COLUMN_KEY));
				String returnedValue = cursor.getString(cursor.getColumnIndex(COLUMN_VALUE));
				long returnedTimestamp = cursor.getLong(cursor.getColumnIndex(COLUMN_TIMESTAMP));
				cursor.close();
				return new SimpleCache(returnedKey, returnedValue, returnedTimestamp);
			} catch (SQLiteException e) {
				Log.w(SimpleCache.class.getName(), "Failed to get readable database: " + e.getMessage());
				return null;
			} finally {
				if (cursor != null) {
					cursor.close();
				}
				db.close();
			}
		}
		
		public boolean replaceSimpleCache(SimpleCache simpleCache) {
			final SQLiteDatabase db;
			try {
				db = getWritableDatabase();
			} catch (SQLiteException e) {
				Log.w(SimpleCache.class.getName(), "Failed to get writeable database: " + e.getMessage());
				return false;
			}
			
			ContentValues cv = new ContentValues();
			cv.put(COLUMN_KEY, simpleCache.mKey);
			cv.put(COLUMN_VALUE, simpleCache.mValue);
			cv.put(COLUMN_TIMESTAMP, simpleCache.mTimestamp);
			
			boolean result = db.replace(TABLE_NAME, null, cv) != -1;
			db.close();
			return result;
		}
	}
}
