/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.location;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.text.TextUtils;
import android.widget.TextView;

import com.iangclifton.android.R;

/**
 * Activity that demonstrates using the device location in a very simple way.
 * 
 * @author Ian G. Clifton
 */
public class LocationActivity extends FragmentActivity implements LocationListener {
	
	private TextView mAddressTextView;
	private boolean mIsReverseGeoding = false;
	private LocationManager mLocationManager;
	private TextView mLocationTextView;
	private String mProvider;
	
	@Override
	public void onLocationChanged(Location location) {
		updateLocationView(location);
	}

	@Override
	public void onProviderDisabled(String provider) {
	}

	@Override
	public void onProviderEnabled(String provider) {
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.location_layout);
		mAddressTextView = (TextView) findViewById(R.id.address);
		mLocationTextView = (TextView) findViewById(R.id.location);
		
		mLocationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		mProvider = LocationManager.GPS_PROVIDER;
		Location location = mLocationManager.getLastKnownLocation(mProvider);
		updateLocationView(location);
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		mLocationManager.removeUpdates(this);
	}

	@Override
	protected void onResume() {
		super.onResume();
		if (mLocationManager.isProviderEnabled(mProvider)) {
			mLocationManager.requestLocationUpdates(mProvider, 0, 0, this);			
		} else {
			mLocationTextView.setText("LocationProvider disabled: " + mProvider);
		}
	}

	/**
	 * Updates the location TextView with data from the new Location
	 * 
	 * @param location Location to populate the TextView with
	 */
	private void updateLocationView(Location location) {
		if (location == null) {
			mLocationTextView.setText("Null Location");
			return;
		}
		StringBuilder sb = new StringBuilder();
		sb.append(location.getProvider())
		  .append(": ")
		  .append(location.getLatitude())
		  .append(',')
		  .append(location.getLongitude())
		  .append(" (")
		  .append(location.getAccuracy())
		  .append(')')
		;
		
		mLocationTextView.setText(sb.toString());
		
		if (!mIsReverseGeoding) {
			mIsReverseGeoding = true;
			new ReverseGeocoderTask().execute(location);
		}
	}


	/**
	 * AsyncTask used to reverse geocode (convert lat/long to address)
	 * 
	 * @author Ian G. Clifton
	 */
	private class ReverseGeocoderTask extends AsyncTask<Location, Void, String> {

		@Override
		protected String doInBackground(Location... locations) {
			Location location = locations[0];
			
			Geocoder geocoder = new Geocoder(LocationActivity.this, Locale.getDefault());
			List<Address> addresses = null;
			try {
				addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1);
			} catch (IOException e) {
				e.printStackTrace();
			}
			
			if (addresses != null && !addresses.isEmpty()) {
				Address address = addresses.get(0);
				return address.getLocality();
			}
			return null;
		}

		@Override
		protected void onPostExecute(String result) {
			if (TextUtils.isEmpty(result)) {
				mAddressTextView.setText("Could not reverse geocode.");
			} else {
				mAddressTextView.setText(result);
			}
		}
		
		
	}
}
