/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.opengl;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;
import android.util.AttributeSet;
import android.view.MotionEvent;

/**
 * Custom GLSurfaceView for rendering a simple pyramid shape.
 * 
 * @author Ian G. Clifton
 */
public class CustomGLSurfaceView extends GLSurfaceView {
	
	private CustomRenderer mCustomRenderer;
	private float mPositionX = 0f;

	public CustomGLSurfaceView(Context context) {
		super(context);
		mCustomRenderer = new CustomRenderer();
		setRenderer(mCustomRenderer);
	}

	public CustomGLSurfaceView(Context context, AttributeSet attrs) {
		super(context, attrs);
		mCustomRenderer = new CustomRenderer();
		setRenderer(mCustomRenderer);
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent e) {
		if (e.getAction() == MotionEvent.ACTION_MOVE) {
			// Handle touch
			float x = e.getX();
			float dx = x - mPositionX;
			if (x < getWidth() / 2) {
				dx *= -1;
			}
			
			mCustomRenderer.mAngle += dx / 5;
			requestRender();
		}
		
		return true;
	}

	/**
	 * GLSurfaceView.Renderer that handles the actual pyramid rendering
	 * 
	 * @author Ian G. Clifton
	 */
	private class CustomRenderer implements GLSurfaceView.Renderer {
		
		private float mAngle = 0f;
		private SimplePyramid mSimplePyramid;

		@Override
		public void onDrawFrame(GL10 gl) {
			gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
			
			// Set view
			gl.glMatrixMode(GL10.GL_MODELVIEW);
			// Load the identity matrix, which is a matrix that does not change
			// anything when multiplied (sort of like "1")
			gl.glLoadIdentity();
			// Adjust our view using the OpenGL Utilities gluLookAt method
			GLU.gluLookAt(gl, 
					0, 0, -4, 
					0, 0, 0, 
					0, 1, 0
			);
			// Rotate the view based on the angle
			gl.glRotatef(mAngle, 0f, 1f, 0f);
			
			// Draw our pyramid
			mSimplePyramid.draw(gl);
		}

		@Override
		public void onSurfaceChanged(GL10 gl, int width, int height) {
			gl.glViewport(0, 0, width, height);
			
			// setup frustum
			float ratio = (float) width / height;
			// Use projection, not orthographic, so objects appear to be 3D
			gl.glMatrixMode(GL10.GL_PROJECTION);
			gl.glLoadIdentity();
			gl.glFrustumf(-ratio, ratio, -1, 1, 1, 10);
		}

		@Override
		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			
			// Clear the whole surface to 0 red, 20% of maximum green, 20% of
			// maximum blue, and 100% opacity
			gl.glClearColor(0f, .2f, .2f, 1f);
			
			// Enable culling of faces for efficiency
			gl.glEnable(GL10.GL_CULL_FACE);
			// Define which faces to cull (the back ones)
			gl.glCullFace(GL10.GL_BACK);
			// Declare the shade model to be smooth for a nicer appearance
			gl.glShadeModel(GL10.GL_SMOOTH);
			// Enable depth testing to draw nearer objects and not those they obscure
			gl.glEnable(GL10.GL_DEPTH_TEST);
			
			gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
			gl.glEnableClientState(GL10.GL_COLOR_ARRAY);
			mSimplePyramid = new SimplePyramid();
		}
		
	}
}
