/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.renderscript;

import com.iangclifton.android.R;

import android.annotation.TargetApi;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.renderscript.Allocation;
import android.renderscript.Matrix4f;
import android.renderscript.ProgramFragment;
import android.renderscript.ProgramFragmentFixedFunction;
import android.renderscript.ProgramStore;
import android.renderscript.ProgramVertex;
import android.renderscript.ProgramVertexFixedFunction;
import android.renderscript.RenderScriptGL;
import android.renderscript.Sampler;

/**
 * RenderScript helper class that makes it easier to work with graphics.
 * 
 * @author Ian G. Clifton
 */
@TargetApi(11)
public class GraphicsRS {
	private RenderScriptGL mRS;
	private ScriptC_Graphics mScript;
	
	private Sampler mSampler;
	private ProgramStore mProgramStore;
	private ProgramVertex mProgramVertex;
	private ProgramFragment mProgramFragment;
	
	public GraphicsRS(RenderScriptGL rs, Resources res, int resId) {
		mRS = rs;
		mScript = new ScriptC_Graphics(rs, res, resId);
		mRS.bindRootScript(mScript);
		
		// Set up image
		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inPreferredConfig = Bitmap.Config.ARGB_8888;
		Bitmap bitmap = BitmapFactory.decodeResource(res, R.drawable.sample_image, options);
		Allocation bitmapAllocation = Allocation.createFromBitmap(mRS, bitmap, Allocation.MipmapControl.MIPMAP_NONE, Allocation.USAGE_GRAPHICS_TEXTURE);
		mScript.set_gImage(bitmapAllocation);
		
		initProgramFragment();
		initProgramStore();
		initProgramVertex();
		initSampler();
	}
	
	/**
	 * Sets the rotation
	 * 
	 * @param rotation float representing the rotation in degrees
	 */
	public void setRotation(float rotation) {
		mScript.set_gRotation(rotation);
	}

	/**
	 * Initializes the (fixed function) ProgramFragment (often known as a
	 * fragment shader).
	 */
	private void initProgramFragment() {
		ProgramFragmentFixedFunction.Builder builder = new ProgramFragmentFixedFunction.Builder(mRS);
		builder.setTexture(ProgramFragmentFixedFunction.Builder.EnvMode.REPLACE, ProgramFragmentFixedFunction.Builder.Format.RGBA, 0);
		mProgramFragment = builder.create();
		mScript.set_gProgramFragment(mProgramFragment);
	}
	
	/**
	 * Initializes the ProgramStore (which controls how the framebuffer is used).
	 * 
	 * Our ProgramStore does no blending or depth testing.
	 */
	private void initProgramStore() {
		mProgramStore = ProgramStore.BLEND_NONE_DEPTH_NONE(mRS);
		mScript.set_gProgramStore(mProgramStore);
	}
	
	/**
	 * Initializes the ProgramVertex (vertext shader)
	 */
	private void initProgramVertex() {
		ProgramVertexFixedFunction.Builder builder = new ProgramVertexFixedFunction.Builder(mRS);
		mProgramVertex = builder.create();
		
		ProgramVertexFixedFunction.Constants constants = new ProgramVertexFixedFunction.Constants(mRS);
		((ProgramVertexFixedFunction) mProgramVertex).bindConstants(constants);
		Matrix4f proj = new Matrix4f();
		proj.loadProjectionNormalized(1, 1);
		constants.setProjection(proj);
		
		mScript.set_gProgramVertex(mProgramVertex);
	}
	
	/**
	 * Initializes the Sampler (used to get pixels from the texture)
	 */
	private void initSampler() {
		mSampler = Sampler.CLAMP_LINEAR(mRS);
		mScript.set_gSampler(mSampler);
	}
}
