/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.sensors;

import android.content.Context;
import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.TextView;

import com.iangclifton.android.R;

/**
 * Activity that displays info about pressure, proximity, and accelerometer.
 * 
 * @author Ian G. Clifton
 */
public class SensorsActivity extends FragmentActivity {

	private float mAccelerometerColorMultiplier;
	private SensorEventListener mAccelerometerListener = new AccelerometerListener();
	private Sensor mAccelerometerSensor;
	private View mAccelerometerView;
	private SensorEventListener mPressureListener = new PressureListener();
	private Sensor mPressureSensor;
	private TextView mPressureTextView;
	private SensorEventListener mProximityListener = new ProximityListener();
	private Sensor mProximitySensor;
	private TextView mProximityTextView;
	private SensorManager mSensorManager;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.sensors);
		mPressureTextView = (TextView) findViewById(R.id.pressure_info);
		mProximityTextView = (TextView) findViewById(R.id.proximity_info);
		mAccelerometerView = findViewById(R.id.accelerometer_info);
		mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
		mProximitySensor = mSensorManager.getDefaultSensor(Sensor.TYPE_PROXIMITY);
		mPressureSensor = mSensorManager.getDefaultSensor(Sensor.TYPE_PRESSURE);
		mAccelerometerSensor = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		float max = mAccelerometerSensor.getMaximumRange();
		mAccelerometerColorMultiplier = 255 / max;
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		if (mProximitySensor != null) {
			mSensorManager.unregisterListener(mProximityListener);
		}
		if (mPressureSensor != null) {
			mSensorManager.unregisterListener(mPressureListener);
		}
		if (mAccelerometerSensor != null) {
			mSensorManager.unregisterListener(mAccelerometerListener);
		}
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		
		if (mProximitySensor != null) {
			mSensorManager.registerListener(mProximityListener, mProximitySensor, SensorManager.SENSOR_DELAY_UI);			
		} else {
			mProximityTextView.setText("No proximity sensor");
		}
		
		if (mPressureSensor != null) {
			mSensorManager.registerListener(mPressureListener, mPressureSensor, SensorManager.SENSOR_DELAY_UI);			
		}
		
		if (mAccelerometerSensor != null) {
			mSensorManager.registerListener(mAccelerometerListener, mAccelerometerSensor, SensorManager.SENSOR_DELAY_UI);			
		}
	}
	
	private class AccelerometerListener implements SensorEventListener {

		@Override
		public void onAccuracyChanged(Sensor sensor, int accuracy) {
			// This notifies you if the accuracy if a sensor has changed
		}

		@Override
		public void onSensorChanged(SensorEvent event) {
			// New accelerometer reading, calculate a color based on X, Y, Z
			
			// Get the values for each axis
			float x = Math.abs(event.values[0]);
			float y = Math.abs(event.values[1]);
			float z = Math.abs(event.values[2]);
			
			// Multiply the value to get it in the 0-255 range
			int r = (int) (x * mAccelerometerColorMultiplier);
			int g = (int) (y * mAccelerometerColorMultiplier);
			int b = (int) (z * mAccelerometerColorMultiplier);
			
			mAccelerometerView.setBackgroundColor(Color.rgb(r, g, b));
		}
		
	}
	
	private class PressureListener implements SensorEventListener {

		@Override
		public void onAccuracyChanged(Sensor sensor, int accuracy) {
			// This notifies you if the accuracy if a sensor has changed			
		}

		@Override
		public void onSensorChanged(SensorEvent event) {
			float millibars = event.values[0];
			// You should use a String resource in a real app
			mPressureTextView.setText(millibars + " millibars of pressure");
		}
		
	}
	
	private class ProximityListener implements SensorEventListener {

		@Override
		public void onAccuracyChanged(Sensor sensor, int accuracy) {
			// This notifies you if the accuracy if a sensor has changed
		}

		@Override
		public void onSensorChanged(SensorEvent event) {
			float cm = event.values[0];
			
			// You should use a String resource in a real app
			mProximityTextView.setText(cm + " cm");
		}
		
	}
}
