/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.view;

import android.content.Context;
import android.content.res.Resources;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.View;

import com.iangclifton.android.R;

/**
 * Custom View used to display primary text, sub-text, and an image.
 * 
 * @author Ian G. Clifton
 */
public class AuthorBox extends View {
	
	private Paint mDefaultPaint = new Paint();
	private int mHeight;
	private Paint mMainTextPaint;
	private String mMainTextString;
	private int mMainTextTop;
	private Paint mSubTextPaint;
	private String mSubTextString;
	private int mSubTextTop;
	private Bitmap mThumbnail = null;
	private int mWidth;

	public AuthorBox(Context context) {
		super(context);
		init(context, null);
	}

	public AuthorBox(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context, attrs);
	}

	public AuthorBox(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		init(context, attrs);
	}
	
	/**
	 * Sets the main text
	 * 
	 * @param mainText String to set the main text to
	 */
	public void setMainText(String mainText) {
		mMainTextString = mainText;
	}
	
	/**
	 * Sets the sub text
	 * 
	 * @param subText String to set the sub text to
	 */
	public void setSubText(String subText) {
		mSubTextString = subText;
	}
	
	/**
	 * Sets the thumbnail
	 * 
	 * @param bitmap Bitmap to set the thumbnail to
	 */
	public void setThumbnail(Bitmap bitmap) {
		mThumbnail = bitmap;
	}
	
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		
		int left = getPaddingLeft();
		int top = getPaddingTop();
		
		if (mThumbnail != null) {
			canvas.drawBitmap(mThumbnail, left, top, mDefaultPaint);
			left += mThumbnail.getWidth();
		}
		
		if (!TextUtils.isEmpty(mMainTextString)) {
			// Note: Complete Views should use Layouts for text.  For text that
			// does not change should use StaticLayout and text that does change
			// should use DynamicLayout.
			canvas.drawText(mMainTextString, left, mMainTextTop, mMainTextPaint);
		}
		
		if (!TextUtils.isEmpty(mSubTextString)) {
			canvas.drawText(mSubTextString, left, mSubTextTop, mSubTextPaint);
		}
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		mHeight = measureHeight(heightMeasureSpec);
		mWidth = measureWidth(widthMeasureSpec);
		mMainTextTop = (int) (getPaddingTop() + mMainTextPaint.getFontSpacing());
		mSubTextTop = (int) (mMainTextTop + mSubTextPaint.getFontSpacing());
		setMeasuredDimension(mWidth, mHeight);
	}

	/**
	 * Initializes the View
	 * 
	 * Note that we do our allocations here.  You should never do any
	 * allocations while measuring or drawing (unless they're conditional, such
	 * as allocating a Paint object the first time {@link #draw(Canvas)} is
	 * called and then re-using it).
	 * 
	 * @param context Context to access app resources
	 * @param attrs AttributeSet for loading main text color
	 */
	private void init(Context context, AttributeSet attrs) {
		setBackgroundResource(R.drawable.appwidget_dark_bg);
		Resources res = context.getResources();
		mMainTextPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		mMainTextPaint.setTextSize(res.getDimension(R.dimen.author_box_main_text));
		mMainTextPaint.setColor(res.getColor(R.color.author_box_text));
		mSubTextPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		mSubTextPaint.setTextSize(res.getDimension(R.dimen.author_box_sub_text));
		mSubTextPaint.setColor(res.getColor(R.color.author_box_text));
		
		// Display something useful for the editor preview
		if (isInEditMode()) {
			setMainText("Main text");
			setSubText("Sub text");
		}
		
		// Read the custom attribute
		if (attrs != null) {
			TypedArray attributes = context.obtainStyledAttributes(attrs, R.styleable.AuthorBoxStyle);
			String color = attributes.getString(R.styleable.AuthorBoxStyle_mainTextColor);
			mMainTextPaint.setColor(Color.parseColor(color));
		}
	}
	
	/**
	 * Calculates the height of the view in DP
	 * @param measureSpec int measure spec for the height
	 * @return the height of the view in DP
	 */
	private int measureHeight(int measureSpec) {
		int result = getPaddingBottom() + getPaddingTop();
		
		int thumbnailHeight = (mThumbnail == null) ? 0 : mThumbnail.getHeight();
		int textHeight = (int) (mMainTextPaint.getFontSpacing() + mSubTextPaint.getFontSpacing());
		
		// Add whichever is bigger, the thumbnail height or the text height, to
		// the padding to determine the full View height
		result += Math.max(thumbnailHeight, textHeight);
		
		return result;
	}
	
	/**
	 * Calculates the width of the view in DP
	 * 
	 * @param measureSpec int measure spec for the width
	 * @return the width of the view in DP
	 */
	private int measureWidth(int measureSpec) {
		int result = 0;
		int specMode = MeasureSpec.getMode(measureSpec);
		int specSize = MeasureSpec.getSize(measureSpec);
		
		if (specMode == MeasureSpec.EXACTLY || specMode == MeasureSpec.AT_MOST) {
			result = specSize;
		} else {
			// We don't handle unspecified widths, but you could by determining
			// either how much you need (e.g., to keep the Strings on a single
			// line) or based on what is visually ideal.
			throw new IllegalStateException("Unsupported spec mode for AuthorBox: " + specMode);
		}		
		return result;
	}
}
