/*
 * Copyright (C) 2012 Ian G. Clifton
 * Used in The Essentials of Android Application Development LiveLessons
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.iangclifton.android.view;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Handler;
import android.service.wallpaper.WallpaperService;
import android.text.format.DateUtils;
import android.view.MotionEvent;
import android.view.SurfaceHolder;

/**
 * Simple WallpaperService that demonstrates drawing and handling touches.
 * 
 * @author Ian G. Clifton
 */
public class MyWallpaperService extends WallpaperService {
	
	private static final long FRAME_TIME = DateUtils.SECOND_IN_MILLIS / 10;
	
	private final Handler mHandler = new Handler();

	@Override
	public Engine onCreateEngine() {
		return new MyEngine();
	}

	/**
	 * This is the Engine that does all the real work.
	 * 
	 * @author Ian G. Clifton
	 */
	private class MyEngine extends Engine {
						
		private Paint mPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		private float mTouchX = -1;
		private float mTouchY = -1;
		private boolean mVisible = false;
		
		private Runnable mRunnable = new Runnable() {
			@Override
			public void run() {
				drawFrame();
			}
		};
		
		private MyEngine() {
			mPaint.setColor(Color.WHITE);
		}

		@Override
		public void onCreate(SurfaceHolder surfaceHolder) {
			super.onCreate(surfaceHolder);
			
			setTouchEventsEnabled(true);
		}

		@Override
		public void onSurfaceChanged(SurfaceHolder holder, int format,
				int width, int height) {
			super.onSurfaceChanged(holder, format, width, height);
			drawFrame();
		}

		@Override
		public void onSurfaceCreated(SurfaceHolder holder) {
			super.onSurfaceCreated(holder);
		}

		@Override
		public void onSurfaceDestroyed(SurfaceHolder holder) {
			super.onSurfaceDestroyed(holder);
			mVisible = false;
		}
		
		@Override
		public void onTouchEvent(MotionEvent event) {
			super.onTouchEvent(event);
			
			if (event.getAction() == MotionEvent.ACTION_MOVE) {
				mTouchX = event.getX();
				mTouchY = event.getY();
				drawFrame();
			} else {
				mTouchX = -1;
				mTouchY = -1;
			}
		}

		@Override
		public void onVisibilityChanged(boolean visible) {
			super.onVisibilityChanged(visible);
			mVisible = visible;
		}
		
		/**
		 * Draws a circle wherever the last touch was and a blue background.
		 */
		private void drawFrame() {
			mHandler.removeCallbacks(mRunnable);
			if (!mVisible) {
				return;
			}
			
			final SurfaceHolder holder = getSurfaceHolder();
			
			Canvas c = null;
			try {
				c = holder.lockCanvas();
				if (c != null) {
					// Draw
					c.drawColor(Color.rgb(0, 0, (int) (System.currentTimeMillis() % 100)));
					c.drawCircle(mTouchX, mTouchY, 50f, mPaint);
				}
			} finally {
				if (c != null) {
					holder.unlockCanvasAndPost(c);
				}
			}
			
			mHandler.postDelayed(mRunnable, FRAME_TIME);
		}
	}
}
